# external dependencies
require 'qiita'

# stdlib dependencies
require 'tmpdir'

module QiitaPicks
  class Generator
    def initialize(params={})
      params[:email]        ||= ''
      params[:password]     ||= ''
      params[:access_token] ||= ''
      @email  = params[:email]
      @mailer = QiitaPicks::Mailer.new(params[:email], params[:password])
      @qiita  = Qiita::Client.new(access_token: params[:access_token])
    end

    def generate_from_url(url=[], opts={})
      opts[:deliver] ||= false
      opts[:deliver] ? generate_and_deliver(url: url) : generate(url: url)
    end

    def generate_from_email_weekly_popular(opts={})
      opts[:limit]   ||= 1
      opts[:deliver] ||= false
      @mailer.weekly_popular(opts[:limit]).each do |mail|
        @basename = mail[:date].strftime QiitaPicks::FILE_BASENAME_EMAIL_WEEKLY_POPULAR
        @subject  = mail[:date].strftime QiitaPicks::EMAIL_SUBJECT_EMAIL_WEEKLY_POPULAR
        opts[:deliver] ? generate_and_deliver(url: mail[:url]) : generate(url: mail[:url])
      end
    end

    def generate_from_email_weekly_stocked(opts={})
      opts[:limit]   ||= 1
      opts[:deliver] ||= false
      @mailer.weekly_stocked(opts[:limit]).each do |mail|
        @basename = mail[:date].strftime QiitaPicks::FILE_BASENAME_EMAIL_WEEKLY_STOCKED
        @subject  = mail[:date].strftime(QiitaPicks::EMAIL_SUBJECT_EMAIL_WEEKLY_STOCKED).gsub(/<SUBJECT>/, mail[:subject])
        opts[:deliver] ? generate_and_deliver(url: mail[:url]) : generate(url: mail[:url])
      end
    end

    def generate_from_list_tag_items(tag, opts={})
      opts[:page]     ||= 1
      opts[:per_page] ||= 20
      opts[:deliver]  ||= false
      @basename = Time.now.strftime QiitaPicks::FILE_BASENAME_LIST_TAG_ITEMS
      @subject  = Time.now.strftime(QiitaPicks::EMAIL_SUBJECT_LIST_TAG_ITEMS).gsub(/<TAG>/, tag.capitalize)
      items = @qiita.list_tag_items(tag, page: opts[:page], per_page: opts[:per_page]).body
      opts[:deliver] ? generate_and_deliver(items: items) : generate(items: items)
    end

    def generate_from_list_user_items(username, opts={})
      opts[:page]     ||= 1
      opts[:per_page] ||= 20
      opts[:deliver]  ||= false
      @basename = Time.now.strftime QiitaPicks::FILE_BASENAME_LIST_USER_ITEMS
      @subject  = Time.now.strftime(QiitaPicks::EMAIL_SUBJECT_LIST_USER_ITEMS).gsub(/<USERNAME>/, username)
      items = @qiita.list_user_items(username, page: opts[:page], per_page: opts[:per_page]).body
      opts[:deliver] ? generate_and_deliver(items: items) : generate(items: items)
    end

    def generate_from_list_user_stocks(username, opts={})
      opts[:page]     ||= 1
      opts[:per_page] ||= 20
      opts[:deliver]  ||= false
      @basename = Time.now.strftime QiitaPicks::FILE_BASENAME_LIST_USER_STOCKS
      @subject  = Time.now.strftime(QiitaPicks::EMAIL_SUBJECT_LIST_USER_STOCKS).gsub(/<USERNAME>/, username)
      items = @qiita.list_user_stocks(username, page: opts[:page], per_page: opts[:per_page]).body
      opts[:deliver] ? generate_and_deliver(items: items) : generate(items: items)
    end

    def generate_from_list_hatebu_popular(opts={})
      opts[:deliver] ||= false
      @basename = Time.now.strftime QiitaPicks::FILE_BASENAME_LIST_HATEBU_POPULAR
      @subject  = Time.now.strftime QiitaPicks::EMAIL_SUBJECT_LIST_HATEBU_POPULAR
      url = QiitaPicks::Picker.hatebu_bookmark_url opts[:tag], :popular
      opts[:deliver] ? generate_and_deliver(url: url) : generate(url: url)
    end

    def generate_from_list_hatebu_recent(opts={})
      opts[:deliver] ||= false
      @basename = Time.now.strftime QiitaPicks::FILE_BASENAME_LIST_HATEBU_RECENT
      @subject  = Time.now.strftime QiitaPicks::EMAIL_SUBJECT_LIST_HATEBU_RECENT
      url = QiitaPicks::Picker.hatebu_bookmark_url opts[:tag], :recent
      opts[:deliver] ? generate_and_deliver(url: url) : generate(url: url)
    end

    private

    def generate(params={})
      params[:url]   ||= []
      params[:items] ||= []
      params[:dest]  ||= '.'
      @basename      ||= Time.now.strftime QiitaPicks::FILE_BASENAME_DEFAULT
      @subject       ||= Time.now.strftime QiitaPicks::EMAIL_SUBJECT_DEFAULT
      @picker = QiitaPicks::Picker.new
      @picker.build_html_from_url(params[:url], @subject)
      @picker.build_html_from_items(params[:items], @subject)
      @picker.save_as_pdf(dest: params[:dest], basename: @basename)
      @picker.save_as_html(dest: params[:dest], basename: @basename)
    end

    def generate_and_deliver(params={})
      Dir.mktmpdir do |dir|
        generate params.merge({dest: dir})
        @mailer.deliver(
          to:        @email,
          subject:   @subject,
          body:      QiitaPicks::EMAIL_CONTENT_DEFAULT,
          add_files: [@picker.path_to_pdf, @picker.path_to_html]
        )
      end
    end
  end
end
